;###########################
;# 1wire bus communication #
;###########################
;
.nolist
.include	"../include/param.inc"
.list
;  ------ 1-Wire Dallas Interface Support ------
	.equ	delay, (CPUCLOCK * 10)/52
	.equ	DS1Wire, 0x90
	;P1.0

	.text
	.using	0

;  ------ DS1Reset ------
.global	DS1Reset
;
;  return Carry if 1-Wire bus not present
;
DS1Reset:
	CLR	DS1Wire		; Start the reset pulse
	MOV	B,#delay
LP1:	NOP
	NOP
	DJNZ	B,LP1		; Wait 480 us with Data low
	CLR	EA		; Interrupt disable
	SETB	DS1Wire		; Release the reset pulse
	; Wait for pullup go high
	MOV	B,#delay	; watchdog
	SETB	C		; DS1820 not present flag
WL1:	JB	DS1Wire,GH1	; Exit loop if line high
	DJNZ	B,WL1		; Hang around if line is low
	SETB	EA		; Interrupt enable
	RET       		; Return if data line is shorted to Low

GH1:	SETB	EA		; Interrupt enable
	MOV	B,#delay	; Presence scanner counter
PR1:	ANL	C,DS1Wire	; Catch presence pulse
	DJNZ	B,PR1		; Scan presence pulse
	RET

;  ------ DS1Rd / DS1Wr ------
.global	DS1Rd
.global	DS1Wr
;
; Shift in/out a byte to the 1Wire, last significant bit first.
;
DS1Rd:	MOV	A,#0xFF
DS1Wr:	MOV	B,#8		;   Bit counter
B81:	CLR	EA		;   Interrupt disable
	CLR	DS1Wire 	;   Start the time slot.
	NOP			;1  Delay
	NOP			;2
	NOP			;3
	RRC 	A      		;4  MOV data bit to carry
	MOV	DS1Wire,C	;5  Send out the data bit
	NOP			;6  Delay
	NOP			;7
	NOP			;8
	NOP			;9
	NOP			;10
	NOP			;11
	MOV	C,DS1Wire	;12 Sample data input (~ 9.8 us / 14.74 MHz)
				;                     (~ 6.5 us / 22.11 MHz)
	SETB	EA		;13 Interrupt enable
	PUSH	B		;15 Save B register
	MOV	B,#(delay/2 - 15);17 Wait for end of time slot
	DJNZ	B,.    		;
	POP	B      		;   Restore B register
	SETB	DS1Wire		;   Terminate time slot
	NOP			;
	DJNZ	B,B81  		;   Bit counter
	RRC	A		;   Save MSB bit
	RET			;   Return to caller.

	;;END
